#!/usr/bin/bash
########################################################################################################
#  -h, --help            list all available options and their default values.
#                        Default values may be set in the configuration file
#                        (/etc/volatilityrc)
#  --conf-file=/home/examiner/.volatilityrc
#                        User based configuration file
#  -d, --debug           Debug volatility
#  --plugins=PLUGINS     Additional plugin directories to use (colon separated)
#  --info                Print information about all registered objects
#  --cache-directory=/home/examiner/.cache/volatility
#                        Directory where cache files are stored
#  --cache               Use caching
#  --tz=TZ               Sets the (Olson) timezone for displaying timestamps
#                        using pytz (if installed) or tzset
#  -f FILEVolatility, --filename=FILENAME
#                        Filename to use when opening an image
#  --profile=WinXPSP2x86
#                        Name of the profile to load (use --info to see a list
#                        of supported profiles)
#  -l LOCATION, --location=LOCATION
#                        A URN location from which to load an address space
#  -w, --write           Enable write support
#  --dtb=DTB             DTB Address
#  --shift=SHIFT         Mac KASLR shift address
#  --output=text         Output in this format (support is module specific, see
#                        the Module Output Options below)
#  --output-file=OUTPUT_FILE
#                        Write output in this file
#  -v, --verbose         Verbose information
#  --physical_shift=PHYSICAL_SHIFT
#                        Linux kernel physical shift address
#  --virtual_shift=VIRTUAL_SHIFT
#                        Linux kernel virtual shift address
#  -g KDBG, --kdbg=KDBG  Specify a KDBG virtual address (Note: for 64-bit
#                        Windows 8 and above this is the address of
#                        KdCopyDataBlock)
#  --force               Force utilization of suspect profile
#  -k KPCR, --kpcr=KPCR  Specify a specific KPCR address
#  --cookie=COOKIE       Specify the address of nt!ObHeaderCookie (valid for
#                        Windows 10 only)
#
#
########################################################################################################


function ProcessArgs()
{
	while true
	do
		case "$1" in
		-h | --help )
			Vargs[${#Vargs[@]}]="$1"
			shift
			;;
		--conf-file )
			ConfFile="$2"
			shift 2
			;;
		-d | --debug )
			Vargs[${#Vargs[@]}]="$1"
			shift
			;;
		--plugins )
			for pd in "$(echo "$2" | tr ":" " ")"
			do
				d="$(dirname "$pd")"
				[[ "$d" == '.' ]] && d="$PWD"
				[[ "$pd" =~ ^/ ]] || pd="$d/$pd"
				DPluginDir="$DPluginDir -v \"$pd:$DPluginDirBaseMP$NPD:ro\""
				Vargs[${#Vargs[@]}]="$([[ $NPD > 0  ]] && echo "$VPluginDir:" || echo "$1=")\"$DPluginDirBaseMP$NPD\""
				((NPD++))
			done
			shift 2
			;;
		--info )
			Vargs[${#Vargs[@]}]="$1"
			shift
			;;
		--cache-directory)
			CacheDir="$(dirname "$2")"
			[[ "$CacheDir" == '.' ]] && CacheDir="$PWD"
			DCacheDir="-v \"$CacheDir:$DCacheDirMP:rw\""
			Vargs[${#Vargs[@]}]="\"$1\" \"$DCacheDirMP/$(basename "$2")\""
			shift 2
			;;
		--cache )
			Vargs[${#Vargs[@]}]="$1"
			shift
			;;
		--tz )
			Vargs[${#Vargs[@]}]="$1"
			Vargs[${#Vargs[@]}]="$2"
			shift 2
			;;
		-f | --filename )
			FilenameDir="$(dirname "$2")"
			[[ "$FilenameDir" == '.' ]] && FilenameDir="$PWD"
			DFilenameDir="-v \"$FilenameDir:$DFilenameMP:rw\""
			Vargs[${#Vargs[@]}]="\"$1\" \"$(basename "$2")\""
			shift 2
			;;
		--profile )
			Vargs[${#Vargs[@]}]="$1"
			Vargs[${#Vargs[@]}]="$2"
			shift 2
			;;
		-l | --location )
			Vargs[${#Vargs[@]}]="$1"
			Vargs[${#Vargs[@]}]="$2"
			shift 2
			;;
		-w | --write )
			Vargs[${#Vargs[@]}]="$1"
			shift
			;;
		--dtb )
			Vargs[${#Vargs[@]}]="$1"
			Vargs[${#Vargs[@]}]="$2"
			shift 2
			;;
		--shift )
			Vargs[${#Vargs[@]}]="$1"
			Vargs[${#Vargs[@]}]="$2"
			shift 2
			;;
		--output )
			Vargs[${#Vargs[@]}]="$1"
			Vargs[${#Vargs[@]}]="$2"
			shift 2
			;;
		--output-file )
			OutputDir="$(dirname "$2")"
			[[ "$OutputDir" == '.' ]] && OutputDir="$PWD"
			DOutputDir="-v \"$OutputDir:$DOutputDirMP:rw\""
			Vargs[${#Vargs[@]}]="$1 \"$DOutputDirMP/$(basename "$2")\""
			shift 2
			;;
		-v | --verbose )
			Vargs[${#Vargs[@]}]="$1"
	                shift
	                ;;
		--physical_shift )
			Vargs[${#Vargs[@]}]="$1"
	                Vargs[${#Vargs[@]}]="$2"
	                shift 2
	                ;;
		--virtual_shift )
			Vargs[${#Vargs[@]}]="$1"
	                Vargs[${#Vargs[@]}]="$2"
	                shift 2
	                ;;
		-g | --kdbg )
			Vargs[${#Vargs[@]}]="$1"
	                Vargs[${#Vargs[@]}]="$2"
	                shift 2
	                ;;
		--force )
			Vargs[${#Vargs[@]}]="$1"
	                shift
	                ;;
		-k | --kpcr )
			Vargs[${#Vargs[@]}]="$1"
	                Vargs[${#Vargs[@]}]="$2"
	                shift 2
	                ;;
		--cookie )
			Vargs[${#Vargs[@]}]="$1"
	                Vargs[${#Vargs[@]}]="$2"
	                shift 2
	                ;;
		--)
			shift
			break
			;;
		*)
			echo Unknown argument - \"$1\" - ignored
			shift
			;;
		esac
	done
	RemainingArgs="$@"
}

################################################################################################################
# Main
################################################################################################################

################################################################################################################
# Constants
################################################################################################################
OS=$(uname -r | sed -n -e 's/.*\(fc..\).*/\1/p' -e 's/.*\(el.\).*/\1/p' -e 's/.*\(amzn.\).*/\1/p')
Vargs=()

Repository=certlifter
Name=volatility
DFilenameMP="/data"
DConfigDirMP="/configs"
DCacheDirMP="/cache]"
DOutputDirMP="/output"
DPluginDirBaseMP="/plugins"
VRC=.volatilityrc
ConfFile="$HOME/$VRC"
NPD=0
ShortOptions=hdf:l:wvq:k:
LongOptions=help,conf-file:,debug,plugins:,info,cache-directory:,cache,tz:,filename:,profile:,location:,write,dtb:,shift:,output:,output-file:,verbose,physical_shift:,virtual_shift:,kdbg:,force,kpcr:,cookie:

case "$OS" in
fc*)
	PROGRAM=docker
	;;
el*)
	PROGRAM=podman
	;;
amzn*)
	PROGRAM=docker
	;;
esac

################################################################################################
# First check that the current user is in the docker group
################################################################################################
case "$OS" in
fc*|amzn*)
	Group=docker
	FakeGroup=test$$
	for group in $(groups) $FakeGroup
	do
		case "$group" in
		$Group)
			break
			;;
		$FakeGroup)
			echo "$USER is not a member of the group $Group - adding $USER to $Group."
			sudo usermod -a -G "$Group" "$USER"
			echo "You must logout and login to activate this new group, then run $0 again."
			;;
		esac
	done
	;;
esac

################################################################################################
# Next, check that docker is running and can find the docker image 
################################################################################################
case "$OS" in
fc*|amzn*)
	Daemon=dockerd
	if ! pidof "$Daemon" > /dev/null 2>&1; then
		echo "Docker is not running. Please start it taking into account your system proxy."
		echo "This site may be helpful: https://docs.docker.com/config/daemon/systemd/#httphttps-proxy"
		echo "Then run $0 again."
		exit 1
	fi
	;;
esac

if ! "$PROGRAM" search "$Repository" 2>&1 | grep -q "$Name"; then
	echo "Cannot find $Repository/$Name."
	echo "Check your proxy and run $0 again."
	exit 1
fi

################################################################################################################
# Fix some known issues
################################################################################################################
case "$OS" in
fc33|fc32)
	D="/sys/fs/cgroup/systemd"
	if ! mount | grep -q "$D" ; then
		echo "Fixing problem on Fedora 33 - see https://github.com/docker/for-linux/issues/841"
		[[ -d $D ]] || sudo mkdir -p "$D"
		sudo mount -t cgroup -o none,name=systemd cgroup /sys/fs/cgroup/systemd
	fi
	;;
esac
################################################################################################################
# Good to go. Process command line args to see if there is a redefinition of the configuration file.
# In either case, process a config file if there is one, translating its contents to command line style first
################################################################################################################
CmdLine="$(getopt -o $ShortOptions --long $LongOptions -- "$@")"
[[ $? != 0 ]] && echo "Error in command line options - Exiting" >&2 && exit 1

if [[ -f "$ConfFile" ]]; then
	RC="$(getopt -o $ShortOptions --long $LongOptions -- $(sed -e '/^\[.*\]$/d' -e 's/\([^=][^=]*\)=\(.*\)/--\L\1\E=\2/' "$ConfFile"))"
	[[ $? != 0 ]] && echo "Error in \"$ConfFile\" - Exiting" >&2 && exit 1
	eval set -- "$RC"
	ProcessArgs "$@"
fi

################################################################################################################
# Next process the command line arguments
################################################################################################################
eval set -- "$CmdLine"
ProcessArgs "$@"

################################################################################################################
# We've collected all of the arguments, so let's run Volatility in the appropriate docker image
################################################################################################################
eval docker run -it --rm			\
			$DConfigDir		\
			$DCacheDir		\
			$DFilenameDir		\
			$DOutputDir		\
			$DPluginDir		\
			$Repository/$Name	\
			vol.py			\
			${Vargs[*]}		\
			"$RemainingArgs"
